/*
 * Copyright (c) 2003, Alexander Greif All rights reserved. (Adapted by Michael
 * J. North for Use in Repast Simphony from Alexander Greifs Flow4J-Eclipse
 * (http://flow4jeclipse.sourceforge.net/docs/index.html), with Thanks to the
 * Original Author) (Michael J. Norths Modifications are Copyright 2007 Under
 * the Repast Simphony License, All Rights Reserved)
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *  * Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer. * Redistributions in
 * binary form must reproduce the above copyright notice, this list of
 * conditions and the following disclaimer in the documentation and/or other
 * materials provided with the distribution. * Neither the name of the
 * Flow4J-Eclipse project nor the names of its contributors may be used to
 * endorse or promote products derived from this software without specific prior
 * written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

package repast.simphony.agents.designer.ui.editors;

import org.eclipse.gef.requests.CreationFactory;
import org.eclipse.gef.tools.ConnectionCreationTool;

import repast.simphony.agents.designer.core.AgentBuilderPlugin;

/**
 * A palette tool for the creation of connections. Has the additional feature,
 * that the default tool is selected if connection tool is activated and the
 * user clicks on a not allowed area.
 * 
 * @author alex
 */
public class AgentBuilderConnectionCreationTool extends ConnectionCreationTool {

	/**
	 * Creates this Tool
	 */
	public AgentBuilderConnectionCreationTool() {
		super();
	}

	/**
	 * Creates this Tool with the given factory
	 * 
	 * @param factory
	 *            the factory to create the parts
	 */
	public AgentBuilderConnectionCreationTool(CreationFactory factory) {
		super(factory);
	}

	/**
	 * If mouse button down without any creation of a connection or without
	 * creating the source connection, then the palette's default tool is
	 * selected.
	 * 
	 * @see org.eclipse.gef.tools.AbstractTool#handleButtonDown(int)
	 */
	@Override
	protected boolean handleButtonDown(int button) {
		boolean result = super.handleButtonDown(button);
		if (button == 1 && !isInState(STATE_CONNECTION_STARTED)
				&& isInState(STATE_INITIAL)) {

			AgentEditor editor = AgentBuilderPlugin.getActiveFlowEditor();
			if (editor != null) {
				editor.activateDefaultTool();
			}
		}
		return result;
	}

}
