/**
 * This file is part of
 * 
 * Copyright (C) 2012 Argonne National Laboratory
 * Copyright (C) 2013 Center for Environmental Systems Research, Kassel, Germany
 *
 * Copyright (C) 2010 Center for Environmental Systems Research, Kassel, Germany
 * 
 * Repast Simphony MySQL Database Outputter is free software; you can redistribute 
 * it and/or modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; If not, see <http://www.gnu.org/licenses/>.
 * 
 * Center for Environmental Systems Research, Kassel
 * Created by Sascha Holzhauer on 13.04.2010
 */
package repast.simphony.data.db.outputter.gui;


import javax.swing.JCheckBox;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JTextField;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;

import org.pietschy.wizard.InvalidStateException;
import org.pietschy.wizard.PanelWizardStep;
import org.pietschy.wizard.WizardModel;

import repast.simphony.data.db.outputter.engine.DbOutputterDescriptor;

import com.jgoodies.forms.factories.DefaultComponentFactory;
import com.jgoodies.forms.factories.FormFactory;
import com.jgoodies.forms.layout.CellConstraints;
import com.jgoodies.forms.layout.ColumnSpec;
import com.jgoodies.forms.layout.FormLayout;
import com.jgoodies.forms.layout.FormSpec;
import com.jgoodies.forms.layout.RowSpec;
import com.jgoodies.forms.layout.Sizes;



/**
 * repast.simphony.data.db
 * 
 * The DB Outputter Wizard Step 1
 * 
 * @author Sascha Holzhauer
 * @date 13.04.2010
 * 
 */
public class DbDataStep extends PanelWizardStep {
	
	/**
	 * The default table name string the text field is initially filled with:
	 */
	protected final String 		DEFAULT_TABLE 		= "tablename";
	
	/**
	 * The default run information table name string the text field is initially filled with:
	 */
	protected final String 		DEFAULT_RUN_INFO_TABLE 		= "runInfoTablename";
	
	/**
	 * The default write cache interval the text field is initially filled with:
	 */
	protected final String		DEFAULT_WRITE_CACHE = "50";
	

	/**
	 * True if a run-ID column shall be added automatically
	 */
	protected final boolean     DEFAULT_ADD_RUNID = true;
		
	
	private static final long		serialVersionUID	= 1L;

	private DbOutputterModel		model;

	// 1st section (data properties):
	private JComponent				separator1;
	private JTextField				nameField;
	private JLabel					nameLabel;
	

	// 2nd section (table columns):
	private JComponent				separator2;
	private JCheckBox				addRunID;
	private JTextField				runInfoTableName;
	private JLabel					runInfoTableLabel;

	// 3rd section: table name
	private JComponent				separator3;
	private JTextField				tableName;


	// 4th section: caching:
	private JComponent				separator4;
	private JTextField				writeCacheInterval;
	private JLabel					writeCacheIntervalLabel;

	/**
	 * Instantiate the 1st step (data step) of the db outputter wizard:
	 */
	public DbDataStep() {
		super("DB Settings", "Enter the run table and cache size.");
		initComponents();
	}

	private void initComponents() {
		DefaultComponentFactory compFactory = DefaultComponentFactory.getInstance();

		// 1st section (data properties):
		separator1 = compFactory.createSeparator("Data Properties");
		nameField = new JTextField();
		nameLabel = new JLabel();


		// 2nd section (table columns):
		separator2 = compFactory.createSeparator("Table Columns");
		addRunID = new JCheckBox("Add run ID", DEFAULT_ADD_RUNID);
		
		
		runInfoTableLabel = new JLabel();
		runInfoTableName = new JTextField(DEFAULT_RUN_INFO_TABLE);

		// 3rd section: table name
		JLabel tableLabel = new JLabel();
		tableName = new JTextField(DEFAULT_TABLE);

		// 4th section: caching:
		separator4 = compFactory.createSeparator("Caching Settings");
		writeCacheInterval = new JTextField(DEFAULT_WRITE_CACHE);
		writeCacheIntervalLabel = new JLabel();

		CellConstraints cc = new CellConstraints();

		setLayout(new FormLayout(new ColumnSpec[] { FormFactory.DEFAULT_COLSPEC,
				FormFactory.LABEL_COMPONENT_GAP_COLSPEC,
				new ColumnSpec(ColumnSpec.FILL, Sizes.DEFAULT, FormSpec.DEFAULT_GROW) }, new RowSpec[] {
				FormFactory.DEFAULT_ROWSPEC, FormFactory.LINE_GAP_ROWSPEC, FormFactory.DEFAULT_ROWSPEC,
				FormFactory.LINE_GAP_ROWSPEC, FormFactory.DEFAULT_ROWSPEC, FormFactory.LINE_GAP_ROWSPEC,
				FormFactory.DEFAULT_ROWSPEC, FormFactory.LINE_GAP_ROWSPEC, FormFactory.DEFAULT_ROWSPEC,
				FormFactory.LINE_GAP_ROWSPEC, FormFactory.DEFAULT_ROWSPEC, FormFactory.LINE_GAP_ROWSPEC,
				FormFactory.DEFAULT_ROWSPEC, FormFactory.LINE_GAP_ROWSPEC, FormFactory.DEFAULT_ROWSPEC,
				FormFactory.LINE_GAP_ROWSPEC, FormFactory.DEFAULT_ROWSPEC, FormFactory.LINE_GAP_ROWSPEC,
				FormFactory.DEFAULT_ROWSPEC, FormFactory.LINE_GAP_ROWSPEC,
				FormFactory.DEFAULT_ROWSPEC, FormFactory.LINE_GAP_ROWSPEC, FormFactory.DEFAULT_ROWSPEC }));
		add(separator1, cc.xywh(1, 1, 3, 1));

		// 1st section (data properties):
		// // ---- nameLabel ----
		nameLabel.setText("Name");
		add(nameLabel, cc.xy(1, 3));

		// ---- nameField ----
		nameField.setText("Database Outputter");
		add(nameField, cc.xy(3, 3));


		nameField.getDocument().addDocumentListener(new DocumentListener() {
			@Override
			public void changedUpdate(DocumentEvent e) {
				updateComplete();
			}

			@Override
			public void insertUpdate(DocumentEvent e) {
				updateComplete();
			}

			@Override
			public void removeUpdate(DocumentEvent e) {
				updateComplete();
			}
		});

		// 2nd section (table columns):
		add(separator2, cc.xywh(1, 9, 3, 1));
		add(addRunID, cc.xy(3, 17));
		runInfoTableLabel.setText("Run-Info Table Name");
		add(runInfoTableLabel, cc.xy(1, 11));
		add(runInfoTableName, cc.xy(3, 11));


		// 3rd section: table name
		separator3 = compFactory.createSeparator("Database Properties");
		add(separator3, cc.xywh(1, 15, 3, 1));
		tableLabel.setText("Table name");
		add(tableLabel, cc.xy(1, 19));
		add(tableName, cc.xy(3, 19));

		// 4th section: caching:
		add(separator4, cc.xywh(1, 21, 3, 1));
		writeCacheIntervalLabel.setText("<html>Set the number of outputs that are<br>cached before sending to DB"
				+ "<br>" + "(Note: every object per tick counts)</html>)");
		add(writeCacheIntervalLabel, cc.xy(1, 23));
		add(writeCacheInterval, cc.xy(3, 23));

		
		tableName.getDocument().addDocumentListener(new DocumentListener() {
			@Override
			public void changedUpdate(DocumentEvent e) {
				updateComplete();
			}

			@Override
			public void insertUpdate(DocumentEvent e) {
				updateComplete();
			}

			@Override
			public void removeUpdate(DocumentEvent e) {
				updateComplete();
			}
		});
		
		
	}

	/**
	 * Called when this wizard step 1 is completed
	 */
	protected void updateComplete() {
		setComplete(
				!nameField.getText().equals("") &&
				!tableName.getText().equals(""));
	}

	/**
	 * @see org.pietschy.wizard.PanelWizardStep#init(org.pietschy.wizard.WizardModel)
	 */
	@Override
	public void init(WizardModel wizardModel) {
		super.init(wizardModel);

		this.model = (DbOutputterModel) wizardModel;
	}

	/**
	 * @see org.pietschy.wizard.PanelWizardStep#prepare()
	 */
	@Override
	public void prepare() {
		super.prepare();

		if (model.getDescriptor() != null && model.getDescriptor().getName() != null) {
			nameField.setText(model.getDescriptor().getName());
		}

		if (model.getDescriptor() != null && model.getDescriptor().getRunInfoTableName() != null) {
			runInfoTableName.setText(model.getDescriptor().getRunInfoTableName());
		}
		
		if (model.getDescriptor() != null && model.getDescriptor().getTableName() != null) {
			tableName.setText(model.getDescriptor().getTableName());
		}

		if (model.getDescriptor() != null) {
			writeCacheInterval.setText(new Integer(model.getDescriptor().getWriteCacheInterval()).toString());
			addRunID.setSelected(model.getDescriptor().isAddRunID());
		}
		updateComplete();
	}

	/**
	 * @see org.pietschy.wizard.PanelWizardStep#applyState()
	 */
	@Override
	public void applyState() throws InvalidStateException {
		
		DbOutputterDescriptor descriptor = model.getDescriptor();
    
	    descriptor.setName(nameField.getText().trim());
    
		super.applyState();

		descriptor.setName(nameField.getText());
		descriptor.setTableName(tableName.getText());
		descriptor.setAddRunID(addRunID.isSelected());
		descriptor.setRunInfoTableName(runInfoTableName.getText());
		
		try {
			descriptor.setWriteCacheInterval(new Integer(writeCacheInterval.getText()).intValue());
		} catch (NumberFormatException e) {
			throw new org.pietschy.wizard.InvalidStateException("Invalid input for cache interval");
		}
	}
}
